import QtQuick 2.15
import QtQuick.Controls 2.15

Window {
    visible: true
    width: 400
    height: 400

    ListModel{
        id:sudokuModel
        property int selectedIndex: -1 // 选中的格子索引
        Component.onCompleted:{
            let board =[
                5, 3, 0, 0, 7, 0, 0, 0, 0,
                6, 0, 0, 1, 9, 5, 0, 0, 0,
                0, 9, 8, 0, 0, 0, 0, 6, 0,
                8, 0, 0, 0, 6, 0, 0, 0, 3,
                4, 0, 0, 8, 0, 3, 0, 0, 1,
                7, 0, 0, 0, 2, 0, 0, 0, 6,
                0, 6, 0, 0, 0, 0, 2, 8, 0,
                0, 0, 0, 4, 1, 9, 0, 0, 5,
                0, 0, 0, 0, 8, 0, 0, 7, 9
            ];
            for (let i = 0; i < 81; i++) {
                sudokuModel.append({ "value": board[i], "isFixed": board[i] !== 0 ,"isError": false,"highlighted": false});
            }
        }
    }

    function checkErrors() {
            let rows = Array(9).fill().map(() => []);
            let cols = Array(9).fill().map(() => []);
            let blocks = Array(9).fill().map(() => []);

            // 先清空所有错误标记
            for (let i = 0; i < 81; i++) {
                sudokuModel.set(i, { value: sudokuModel.get(i).value,isFixed:sudokuModel.get(i).isFixed, isError: false });
            }

            // 遍历数独，存储每个格子的索引
            for (let j = 0; j < 81; j++) {
                let val = sudokuModel.get(j).value;
                if (val === 0) continue; // 跳过空格

                let row = Math.floor(j / 9);
                let col = j % 9;
                let block = Math.floor(row / 3) * 3 + Math.floor(col / 3);

                // 记录数字出现的位置
                rows[row].push({ val, index: j });
                cols[col].push({ val, index: j });
                blocks[block].push({ val, index: j });
            }

            // 标记重复项
            function markDuplicates(arr) {
                let seen = {};
                arr.forEach(group => {
                    seen = {}; // 重置
                    group.forEach(cell => {
                        if (seen[cell.val]) {
                            // 发现重复项，标记错误
                            sudokuModel.set(cell.index, { value: cell.val, isError: true });
                            sudokuModel.set(seen[cell.val], { value: cell.val, isError: true });
                        } else {
                            seen[cell.val] = cell.index;
                        }
                    });
                });
            }

            // 标记错误格子
            markDuplicates(rows);
            markDuplicates(cols);
            markDuplicates(blocks);
        }



    Grid {
        anchors.centerIn: parent
        columns: 9
        spacing: 2

        Repeater {
            model: sudokuModel
            delegate: Rectangle {
                width: 40
                height: 40
                border.color: "black"
                color: model.isError ? "red":(model.highlighted?"lightgray":"white")

                property bool isFocused: false  // 用于管理选中状态

                Text {
                    anchors.centerIn: parent
                    text: model.value !== 0 ? model.value : ""
                    font.bold: true
                }

                MouseArea {
                    anchors.fill: parent
                    acceptedButtons: Qt.LeftButton | Qt.RightButton
                    onClicked: {
                        sudokuModel.selectedIndex = index;
                        if (mouse.button === Qt.RightButton) { // 'mouse' is a MouseEvent argument passed into the onClicked signal handler
                            if (sudokuModel.get(index).isFixed===false)
                                sudokuModel.get(index).value=0;
                                checkErrors();
                        } else if (mouse.button === Qt.LeftButton) {
                            if (sudokuModel.get(index).isFixed===false){
                                parent.forceActiveFocus(); // 让当前 Rectangle 获得焦点
                                parent.isFocused = true;
                                updateHighlighting();
                            }else{
                                parent.forceActiveFocus(); // 让当前 Rectangle 获得焦点
                                parent.isFocused = false;
                            }
                        }
                    }
                }

                Keys.onPressed: (event) => {
                                    if (isFocused) {
                                        let newValue = model.value;
                                        switch (event.key) {
                                            case Qt.Key_Up:
                                            newValue = Math.min(model.value + 1, 9);
                                            break;
                                            case Qt.Key_Down:
                                            newValue = Math.max(model.value - 1, 0);
                                            break;
                                        }
                                        sudokuModel.set(index, { value: newValue, isError: false }); // 先更新数值
                                        checkErrors(); // 再检查错误
                                    }
                                }
            }
        }
    }

    function updateHighlighting() {
            if (sudokuModel.selectedIndex === -1) return;

            let selectedRow = Math.floor(sudokuModel.selectedIndex / 9);
            let selectedCol = sudokuModel.selectedIndex % 9;
            let selectedBlock = Math.floor(selectedRow / 3) * 3 + Math.floor(selectedCol / 3);

            for (let i = 0; i < 81; i++) {
                let row = Math.floor(i / 9);
                let col = i % 9;
                let block = Math.floor(row / 3) * 3 + Math.floor(col / 3);

                let shouldHighlight = (row === selectedRow || col === selectedCol || block === selectedBlock);
                sudokuModel.set(i, {
                    highlighted: shouldHighlight});
            }
    }




    function checkwin(){
        for (let i = 0; i < 81; i++) {
            if (sudokuModel.get(i).value===0 || sudokuModel.get(i).isError===true) {
                console.log("C'est mort ~_~")
                return;
        }
        console.log("Il a gagné!")
        }
    }
}

